from typing import Optional, List
from http import HTTPStatus
from pydantic import BaseModel, Field, ValidationError
from pydantic.schema import schema
import json
import uuid, shortuuid
import random, hashlib
from datetime import datetime, timezone, timedelta
from opsramp_domain_model.models.AlertModel import Alert, PostedAlert
from opsramp_api_utils import utils



class AlertArchetype():
    subject = [
        "SNMP request timeout on 000C2923548D",
        "generic subject"
    ]
    description = [
        "SNMP requests are timed out for the following reasons...",
        "generic description"
    ]
    metric = [
        "snmp.response.timeout",
        "generic metric"
    ]
    state = [
        "ok", "warning", "critical"
    ]
    status = [
        "open", "acknowleged", "suppressed", "ticketed", "closed"
    ]

    attributes = {}

    def __init__(self):
        self.attributes["id"] = str(random.randrange(100000000, 200000000))
        self.attributes["subject"] = random.choice(self.subject)
        self.attributes["description"] = random.choice(self.description)
        self.attributes["metric"] = random.choice(self.metric)
        self.attributes["alertType"] = "monitoring"
        self.attributes["state"] = random.choice(self.state)
        self.attributes["status"] = random.choice(self.status)
        self.attributes["objectId"] = str(uuid.uuid4())
        self.attributes["inferenceId"] = str(random.randrange(100000000, 200000000))
        self.attributes["incidentId"] = "INC" + str(random.randrange(1000000000, 2000000000))

        refTime = datetime.fromisoformat('2020-01-01T00:00:00+00:00')
        refTime_epoch = int(round(refTime.timestamp()))
        #refTime_epoch = utils.get_epoch_time(refTime)

        triggeredTime_epoch = refTime_epoch + random.randrange(0, 60*60*24*30)
        createdTime_epoch = triggeredTime_epoch + 10
        updatedTime_epoch = createdTime_epoch + random.randrange(60, 60*60)

        self.attributes["triggeredTime"] = \
            datetime.fromtimestamp(triggeredTime_epoch, tz=timezone.utc).strftime("%y-%m-%dT%H:%M:%S") + "+0000"
        self.attributes["createdTime"] = \
            datetime.fromtimestamp(createdTime_epoch, tz=timezone.utc).strftime("%y-%m-%dT%H:%M:%S") + "+0000"
        self.attributes["updatedTime"] = \
            datetime.fromtimestamp(updatedTime_epoch, tz=timezone.utc).strftime("%y-%m-%dT%H:%M:%S") + "+0000"

    def generate_examples(count: int):

        alerts = []
        alert = None
        for c in range(count):
            #print("c:", c)
            attributes = AlertArchetype().attributes
            #print("attributes:", json.dumps(attributes, indent=4))
            #alert = (Alert(**attributes)).dict()
            #print("type of alert:", type(alert))
            #print(alert)

            
            try:
                alert = (Alert(**attributes)).dict()
                #print("type of alert:", type(alert))
                #print(alert)
            except ValidationError as e:
                print("model validation error", e.json())
            

            alerts.append(alert)

        return alerts




def get_alerts():

    return alerts


# intial pool size
INITIAL_SAMPLE_SIZE = 4
# see a pool of synthetically generated resources
alerts = AlertArchetype.generate_examples(INITIAL_SAMPLE_SIZE)



if __name__ == "__main__":
    CANONICAL_ALERT_FILENAME = './canonical-alert.json'
    utils.print_dict_to_file(alerts[0], CANONICAL_ALERT_FILENAME, sort=False)
    pass

