from typing import Optional, List
from pydantic import BaseModel, Field, ValidationError
from pydantic.schema import schema
import json
import uuid, shortuuid
import random, hashlib
from datetime import datetime, timezone, timedelta
from opsramp_domain_model.models.ResourceModel import Resource, DiscoveredResource
#from ..models.ResourceModel import Resource, DiscoveredResource



def get_resource_uuid():
    new_uuid = str(uuid.uuid4())

    return new_uuid


class ResourceArchetype():
    make_values = [ "Dell, Inc.", "VMware, Inc.", "AWS", "Azure" ]
    model_values = [ "Model-A", "Model-B", "Model-C", "Model-D" ]
    attributes = {}

    def __init__(self):
        self.attributes["id"] = get_resource_uuid()
        self.attributes["type"] = "server"
        self.attributes["nativeType"] = "server"
        self.attributes["name"] = "res" + "-" + str(shortuuid.uuid())
        self.attributes["aliasName"] = self.attributes["name"] + "-" + "alias"
        self.attributes["dnsName"] = self.attributes["name"] + ".subdomain" + ".domain.com"
        self.attributes["ipAddress"] = "10.0.0." + str(random.randrange(2,255))
        self.attributes["alternateIpAddress"] = "172.0.0." + str(random.randrange(2, 255))

        num_digits_in_mac_address = 12
        random_hex_num = lambda n: ((hex(random.randrange(0, 15))).split("x"))[1]
        mac_address_digits = [ random_hex_num(n) for n in range(num_digits_in_mac_address) ]

        mac_address = ""
        for i in range(int(num_digits_in_mac_address/2)):
            if i == 0:
                mac_address = mac_address + str(mac_address_digits[i]) +  str(mac_address_digits[i])
            else:
                mac_address = mac_address + ":" + str(mac_address_digits[i]) + str(mac_address_digits[i])

        self.attributes["macAddress"] =  mac_address

        self.attributes["os"] = "Ubuntu 20.04 LTS"
        self.attributes["make"] = random.choice(self.make_values)
        self.attributes["model"] = random.choice(self.model_values)
        self.attributes["serialNumber"] = str(random.randrange(10000, 20000))

        h1 = hashlib.sha256()
        h1.update(bytes(self.attributes["serialNumber"], 'utf-8'))

        h2 = hashlib.sha256()
        h2.update(bytes(self.attributes["macAddress"], 'utf-8'))

        self.attributes["signatures"] = [ h1.hexdigest(), h2.hexdigest() ]
        self.attributes["state"] = "managed"

    def generate_examples(count : int):

        resources = []
        for c in range(count):
            attributes = ResourceArchetype().attributes


            resource = None
            try:
                resource = (Resource(**attributes)).dict()
            except ValidationError as e:
                print (e.json())

            resources.append(resource)

        return resources


def get_object_types():
    object_types = [
        {
            "object": Resource.__name__, 
            "description" : "represents the managed resource in OpsRamp"
        },
        { 
            "object": Alert.__name__,
            "description" : "represents the alert ingested or generated OpsRamp"
        },

    ]

    return object_types


def get_json_schema():
    json_schema = schema([Resource, Alert], title='OpsRamp Domain Model Schema')

    return json_schema



def get_resources():

    return resources


def create_new_resource(discovered_resource : DiscoveredResource):
    attributes = discovered_resource.dict()
    attributes["id"] = get_resource_uuid()
    attributes["state"] = "managed"
    resource = Resource(**attributes)

    return resource

def lookup_resource(discovered_resource : DiscoveredResource):
    
    #print("type discovered_resource:", type(discovered_resource))
    #print(json.dumps(discovered_resource.dict(), indent=4))
    resource = None

    for existing_resource in resources:
        #print("type existing resource", type(existing_resource))
        #print("existing resource", existing_resource)
        if (discovered_resource.dict())["moId"] == existing_resource["moId"]:
            resource = existing_resource
            #print("resource lookup", resource.dict())

    return resource


def add_to_resource_pool(resource: Resource):
    resources.append(resource)

    return


def post_resource(discovered_resource: DiscoveredResource):

    
    created_new_resource = False
    #print(discovered_resource.dict())
    resource = lookup_resource(discovered_resource)
    if resource == None:
        new_resource = create_new_resource(discovered_resource)
        add_to_resource_pool(new_resource)
        created_new_resource = True
    else:
        resource.update(discovered_resource)

    return created_new_resource



# intial pool size
INITIAL_SAMPLE_SIZE = 4
# see a pool of synthetically generated resources
resources = ResourceArchetype.generate_examples(INITIAL_SAMPLE_SIZE)


if __name__ == "__main__":
    from opsramp_api_utils import utils
    CANONICAL_RESOURCE_FILENAME = './canonical-resource.json'
    utils.print_dict_to_file(resources[0], CANONICAL_RESOURCE_FILENAME, sort=False)
    pass

